from unittest.mock import MagicMock, patch

from revChatGPTAuth.chatgpt_access_token_parser import ChatGPTAccessTokenParser, get_access_token


class TestChatGPTAccessTokenParser:

    @patch('revChatGPTAuth.chatgpt_access_token_parser.OpenAICookieParser')
    @patch('httpx.get')
    def test_get_openai_chatgpt_access_token(
        self,
        mock_httpx_get: MagicMock,
        mock_openai_cookie_parser: MagicMock,
    ) -> None:
        # given
        ## mock cookies
        mock_cookie_parser = MagicMock()
        mock_cookie_parser.parse_cookie.return_value = {'cookie_key': 'cookie_value'}
        mock_openai_cookie_parser.return_value = mock_cookie_parser

        ## mock access token response
        expected_access_token = 'access_token'
        mock_response = MagicMock()
        mock_response.json.return_value = {'accessToken': expected_access_token}
        mock_httpx_get.return_value = mock_response

        chatgpt_access_token_parser = ChatGPTAccessTokenParser('chrome')

        # when
        access_token = chatgpt_access_token_parser.get_openai_chatgpt_access_token()

        # then
        mock_cookie_parser.parse_cookie.assert_called_once()
        mock_httpx_get.assert_called_once_with(
            'https://chat.openai.com/api/auth/session',
            headers={'cookie': 'cookie_key=cookie_value'},
        )
        mock_response.raise_for_status.assert_called_once()
        assert access_token == expected_access_token


@patch('revChatGPTAuth.chatgpt_access_token_parser.ChatGPTAccessTokenParser')
def test_get_access_token(mock_chatgpt_access_token_parser: MagicMock) -> None:
    # given
    expected_access_token = 'access_token'
    mock_chatgpt_access_token_parser.return_value.get_openai_chatgpt_access_token.return_value = expected_access_token

    # when
    access_token = get_access_token('chrome')

    # then
    mock_chatgpt_access_token_parser.assert_called_once_with('chrome')
    assert access_token == expected_access_token



from http.cookiejar import Cookie
from unittest.mock import MagicMock, patch

import pytest
from yt_dlp.cookies import SUPPORTED_BROWSERS
from yt_dlp.utils import YoutubeDLCookieJar

from revChatGPTAuth.openai_cookie_parser import OpenAICookieParser


class TestOpenAICookieParser:

    def test_init_with_unsupported_browser(self):
        with pytest.raises(ValueError):
            OpenAICookieParser(browser_name='Internet Explorer Holy Shit')

    @pytest.mark.parametrize('supported_browser', SUPPORTED_BROWSERS)
    def test_init_with_supported_browser(self, supported_browser: str):
        parser = OpenAICookieParser(browser_name=supported_browser)
        assert parser.BROWSER_NAME == supported_browser

    @pytest.mark.parametrize('supported_browser', SUPPORTED_BROWSERS)
    @patch('revChatGPTAuth.openai_cookie_parser.extract_cookies_from_browser')
    def test_parse_cookie_with_supported_browser(
        self,
        mock_extract_cookies: MagicMock,
        supported_browser: str,
    ):
        # given
        cookie_jar = YoutubeDLCookieJar()
        cookie_jar.set_cookie(
            Cookie(
                version=0,
                name='cookie1',
                value='value1',
                port=None,
                port_specified=False,
                domain='chat.openai.com',
                domain_specified=True,
                domain_initial_dot=False,
                path='/',
                path_specified=True,
                secure=False,
                expires=None,
                discard=True,
                comment=None,
                comment_url=None,
                rest={},
            ))
        cookie_jar.set_cookie(
            Cookie(
                version=0,
                name='cookie2',
                value='value2',
                port=None,
                port_specified=False,
                domain='.openai.com',
                domain_specified=False,
                domain_initial_dot=True,
                path='/',
                path_specified=True,
                secure=False,
                expires=None,
                discard=True,
                comment=None,
                comment_url=None,
                rest={},
            ))
        cookie_jar.set_cookie(
            Cookie(
                version=0,
                name='cookie3',
                value='value3',
                port=None,
                port_specified=False,
                domain='.chat.openai.com',
                domain_specified=False,
                domain_initial_dot=True,
                path='/',
                path_specified=True,
                secure=False,
                expires=None,
                discard=True,
                comment=None,
                comment_url=None,
                rest={},
            ))
        mock_extract_cookies.return_value = cookie_jar
        parser = OpenAICookieParser(browser_name=supported_browser)

        # when
        openai_cookies = parser.parse_cookie()

        # then
        assert openai_cookies == {'cookie1': 'value1', 'cookie2': 'value2', 'cookie3': 'value3'}



from http.cookiejar import Cookie
from typing import Any, Optional

from yt_dlp.cookies import SUPPORTED_BROWSERS, extract_cookies_from_browser
from yt_dlp.utils import YoutubeDLCookieJar


class OpenAICookieParser:

    def __init__(self, browser_name: str):
        if browser_name not in SUPPORTED_BROWSERS:
            raise ValueError(f'Browser {browser_name} is not supported. Supported browsers are: {SUPPORTED_BROWSERS}')
        self.BROWSER_NAME = browser_name

    def parse_cookie(self):
        all_cookies: YoutubeDLCookieJar = extract_cookies_from_browser(self.BROWSER_NAME)
        openai_cookies = self._get_openai_cookies(all_cookies.__dict__)
        openai_cookies_dict: dict[str, Optional[str]] = {key: cookie.value for key, cookie in openai_cookies.items()}
        return openai_cookies_dict

    def _get_openai_cookies(self, all_cookies_dict: dict[str, Any]):
        try:
            chat_openai_cookies: dict[str, Cookie] = all_cookies_dict['_cookies']['chat.openai.com']['/']
            dot_chat_openai_cookies: dict[str, Cookie] = all_cookies_dict['_cookies']['.chat.openai.com']['/']
            dot_openai_cookies: dict[str, Cookie] = all_cookies_dict['_cookies']['.openai.com']['/']
            openai_cookies = {**chat_openai_cookies, **dot_chat_openai_cookies, **dot_openai_cookies}
            return openai_cookies
        except KeyError as error:
            raise ValueError(
                f'OpenAI cookies not found. Please log in to OpenAI using your browser {self.BROWSER_NAME} to proceed.'
            ) from error



from .chatgpt_access_token_parser import ChatGPTAccessTokenParser as ChatGPTAccessTokenParser
from .chatgpt_access_token_parser import get_access_token as get_access_token
from .openai_cookie_parser import OpenAICookieParser as OpenAICookieParser



import httpx

from .openai_cookie_parser import OpenAICookieParser


class ChatGPTAccessTokenParser:

    def __init__(self, browser_name: str):
        self.openai_cookie_parser = OpenAICookieParser(browser_name)

    def get_openai_chatgpt_access_token(self) -> str:
        cookie = self._get_stringified_cookies()
        response = self._auth_openai(cookie)
        response.raise_for_status()
        return response.json()['accessToken']

    def _get_stringified_cookies(self) -> str:
        cookies = self.openai_cookie_parser.parse_cookie()
        return '; '.join([f'{key}={value}' for key, value in cookies.items()])

    def _auth_openai(self, cookie: str) -> httpx.Response:
        URL = 'https://chat.openai.com/api/auth/session'
        headers = {'cookie': cookie}
        response = httpx.get(URL, headers=headers)
        return response


def get_access_token(browser_name: str) -> str:
    chatgpt_access_token_parser = ChatGPTAccessTokenParser(browser_name)
    return chatgpt_access_token_parser.get_openai_chatgpt_access_token()
